######################################################################################
######################################################################################

# Example of an R-code to fit a linear model with the library BRugs

# Compare a linear regression and a linear regression with residuals ~ AR1

# E. RIVOT & E. PARENT - June 2014

######################################################################################
######################################################################################


rm(list=ls())
library(BRugs)
library(MASS)


# load a function needed to save and plot mcmc results
source("f.save.mcmc.R")
source("f.panel.cor.R")
source("f.panel.dens.R")
source("f.density.bivar.R")

dir.create(path = c("res"), recursive = TRUE, showWarnings = TRUE)


# ------------------------------------------------------------------------------------
# MCMC setting
# ------------------------------------------------------------------------------------

# Number iteration for the "burn in" phase (MCMC sample not used for inference)
n.burnIn <- 2000
# Number iteration for inferences
n.iter <- 2000
thin <- 1
# Number of chains
n.Chains <- 3



# ------------------------------------------------------------------------------------
# Run model
# ------------------------------------------------------------------------------------

# Load/check model, load data and inits

model<-"model_reg_BUGS.txt"
data <- "data.txt"
data.2002 <- "data.2002.txt"
inits <- "inits.txt"

modelCheck(model) 
modelData(data) 
modelData(data.2002) 
modelCompile(numChains=n.Chains) 
modelInits(c(inits,inits,inits))
modelGenInits()


# Monitored variables

monitor <- c("a", "a.p", "b.2", "tau", "rho.fit", "rho.pred", "p.value", "res", "Tw.2002.pred")

# Burn in period - MCMC sampels not used for inference

modelUpdate(numUpdates = n.burnIn, thin = 1) 

# MCMC samples used for inference

samplesSet(monitor) 
dicSet()
modelUpdate(numUpdates = n.iter, thin = thin) 





# -----------------------------------------------------------
# Info MCMC updater method
# -----------------------------------------------------------

infoUpdatersbyName()



# -------------------------------------------------------------------------
# History MCMC chains
# -------------------------------------------------------------------------

Results <- samplesHistory("a")

windows()
samplesHistory("*",mfrow = c(3, 2))



# -----------------------------------------------------------
# Plot de Gelman-Rubin BGR ratio
# -----------------------------------------------------------

windows()
samplesBgr("*", mfrow = c(3, 2))

# return the value of the Bgr ratio
samplesBgr("a", plot=FALSE)



# -------------------------------------------------------------------------
# Posterior densities
# -------------------------------------------------------------------------

windows()
samplesDensity("*", mfrow = c(3, 3)) 



# -------------------------------------------------------------------------
# Posterior statistics
# -------------------------------------------------------------------------

stats <- samplesStats("*")
stats

# save in a table
setwd("./res")
name <- "stats"
write.table(x=stats, file = name, append = F, quote = F)
setwd("../")



# -----------------------------------------------------------
# Get MCMC samples
# -----------------------------------------------------------

tau <- samplesSample("tau")
sigma <- tau^(-0.5)
a <- samplesSample("a")
a.p <- samplesSample("a.p")
b.2 <- samplesSample("b.2")
rho.fit <- samplesSample("rho.fit")
rho.pred <- samplesSample("rho.pred")
p.value <- samplesSample("p.value")

# NOTE
# for multidimentional variables, use the function "build.mcmc"

mcmc.res <- as.data.frame(as.matrix(buildMCMC("res")))
windows()
boxplot(mcmc.res, outline = FALSE)

mcmc.Tw.2002.pred <- as.data.frame(as.matrix(buildMCMC("Tw.2002.pred")))



# -------------------------------------------------------------------------
# Save / Read MCMC samples
# -------------------------------------------------------------------------

setwd("./res")

for(i in 1:length(monitor)) {f.save.mcmc(var=monitor[i],write=TRUE)}

# a <- as.vector(read.table(file = "a", header = T, sep = ""))$a
# b.2 <- as.vector(read.table(file = "b.2", header = T, sep = ""))$b

setwd("../")



# -------------------------------------------------------------------------
# DIC
# -------------------------------------------------------------------------

DIC <- dicStats()
DIC

# Save in a Table
setwd("./res")
name <- "DIC"
write.table(x=DIC, file = name, append = F, quote = F)
setwd("../")



# --------------------------------------------------------
# Draw marginal posterior density (prior / posterior)
# --------------------------------------------------------

x <- a
y <- a.p

windows()
cise.text <- 1.5
par(mfrow = c(1,1), oma = c(1,1,1,1))

plot(x=0, y=-1, ylim = c(0,max(density(x)$y)*1.2), xlim = c(0,1), ylab = "", xlab = "", bty="n")
lines(density(x), xlim = c(0,1), col = "black", lty = 1, lwd = 2)
lines(density(y), xlim = c(0,1), col = "black", lty = 2, lwd = 2)

mtext(side=3, text = expression(paste("Prior and posterior distribution of ",a)), bty="n", line=1, cex=1.5, font = 1)
mtext(side=1, text = "a", bty="n", line=3, cex=cise.text, font = 1)
mtext(side=2, text = "Density", bty="n", line=3, cex=cise.text, font = 1)

legend(	legend = c("Prior","Posterior"),
 		col = c("black","black"), 	
 		lty = c(2,1), lwd = c(2,2), x = "topright", cex = 1, bty ="n") 



# ------------------------------------------------------------
# Marginal posterior and correlations
# ------------------------------------------------------------

x1 <- as.vector(a)
x2 <- as.vector(b.2)
x3 <- as.vector(sigma)

def.par <- par(no.readonly = TRUE)
windows()

par(pch='.')

pairs( cbind(x1,x2,x3),labels=c("a","b",expression(sigma)), 
	 lower.panel=panel.smooth,
	 diag.panel=panel.dens, 
	 upper.panel=panel.cor,
	 cex.labels = 2.0, font.labels=1.2)



# -----------------------------------------------------
# Joint posterior distribution for (a,b.2)
# -----------------------------------------------------

x <- a
y <- b.2

def.par <- par(no.readonly = TRUE)
windows()

f.density.bivar(x,y,nlevels=3,nb.points=2000)
par(def.par)



# -----------------------------------------------------
# Visualize parameter uncertainty in the regression
# -----------------------------------------------------


Ta <- source(data)$value$Ta
Tw <- source(data)$value$Tw

def.par <- par(no.readonly = TRUE)
windows()
par(pch='.')

plot(Ta,Tw, xlab = "Tair", ylab = "Twater", bty="n", pch=".", cex=2)
nb.line <- 10
for (i in 1:nb.line)
{
abline(b.2[i],a[i],col="grey",lwd=0.1)
}



# -----------------------------------------------------
# Posterior predictive check
# -----------------------------------------------------


x <- rho.pred
y <- rho.fit

def.par <- par(no.readonly = TRUE)
windows()
par(pch='.')

plot(x,y, xlab = "pred", xlim = c(-0.5,0.5), ylim=c(-0.5,0.5), ylab = "fit", main = "post. predictive check ; statistics = rho", bty="n")
abline(a=0,b=1,col="red")
text( x=0, y=0, pos=4, as.expression( substitute( p.value==p.val,
      list(p.val=mean(p.value)))))


# ----------------------------------------------------------------------------
#                        Predictions of Tw in 2002
# ----------------------------------------------------------------------------
			
n.2002 <- source("data.2002.txt")$value$n.2002
Tw.2002 <- source("data.2002.txt")$value$Tw.2002
			
time <- 1:n.2002
time.inv <- n.2002:1

X2 <- mcmc.Tw.2002.pred

med <- boxplot(X2)$stats[3,]
conf1l <- boxplot(X2)$stats[1,]
conf1u <- boxplot(X2)$stats[5,]
conf2l <- boxplot(X2)$stats[2,]
conf2u <- boxplot(X2)$stats[4,]

windows()

plot(	time, Tw.2002, type = "n", col = "blue", ylim = c(10,22), lwd = 3, main = "Prediction TC Air - Summer 2002")

polygon(c(time,time.inv),c(conf1l,conf1u[n.2002:1]),col="grey80",border=NA)
polygon(c(time,time.inv),c(conf2l,conf2u[n.2002:1]),col="grey60",border=NA)

points(time, med, type = "l", col = "black", lwd=2, lty=1)

points( time, Tw.2002, type = "l", col = "blue", lwd = 3)





# -----------------------------------------------------
# -----------------------------------------------------
#  Compare with or without AR1
# -----------------------------------------------------
# -----------------------------------------------------


n.burnIn <- 1000
n.iter <- 1000
thin <- 1
n.Chains <- 3

# Load/check model, load data and inits

model<-"model_reg_ar1_BUGS.txt"
modelCheck(model) 
modelData(data) 
modelCompile(numChains=n.Chains) 
modelInits(c(inits,inits,inits))
modelGenInits()

monitor <- c("a", "b.2", "tau", "rho", "rho.fit")

# modelUpdate(numUpdates = n.burnIn, thin = 1) 
# samplesSet(monitor) 
# dicSet()
# modelUpdate(numUpdates = n.iter, thin = thin) 

DIC <- dicStats()
DIC

a.ar1 <- samplesSample("a")
b.2.ar1 <- samplesSample("b.2")
rho.fit.ar1 <- samplesSample("rho.fit")
rho <- samplesSample("rho")


# --------------------------------------------------------
# Draw marginal posterior density reg / reg with ar1
# --------------------------------------------------------

x <- a
y <- a.ar1

windows()
cise.text <- 1.5
par(mfrow = c(1,1), oma = c(1,1,1,1))

plot(x=0, y=-1, ylim = c(0,max(density(x)$y)*1.2), xlim = c(0,1), ylab = "", xlab = "", bty="n")
lines(density(x), xlim = c(0,1), col = "black", lty = 1, lwd = 2)
lines(density(y), xlim = c(0,1), col = "black", lty = 2, lwd = 2)

mtext(side=3, text = expression(paste("Posterior distribution of ",a)), bty="n", line=1, cex=1.5, font = 1)
mtext(side=1, text = "a", bty="n", line=3, cex=cise.text, font = 1)
mtext(side=2, text = "Density", bty="n", line=3, cex=cise.text, font = 1)

legend(	legend = c("reg lin","reg lin with AR1"),
 		col = c("black","black"), 	
 		lty = c(2,1), lwd = c(2,2), x = "topright", cex = 1, bty ="n") 


x <- b.2
y <- b.2.ar1

windows()
cise.text <- 1.5
par(mfrow = c(1,1), oma = c(1,1,1,1))

plot(x=0, y=-1, ylim = c(0,max(density(x)$y)*1.2), xlim = c(0,20), ylab = "", xlab = "", bty="n")
lines(density(x), xlim = c(0,20), col = "black", lty = 1, lwd = 2)
lines(density(y), xlim = c(0,20), col = "black", lty = 2, lwd = 2)

mtext(side=3, text = expression(paste("Posterior distribution of ",b)), bty="n", line=1, cex=1.5, font = 1)
mtext(side=1, text = "b", bty="n", line=3, cex=cise.text, font = 1)
mtext(side=2, text = "Density", bty="n", line=3, cex=cise.text, font = 1)

legend(	legend = c("reg lin","reg lin with AR1"),
 		col = c("black","black"), 	
 		lty = c(2,1), lwd = c(2,2), x = "topright", cex = 1, bty ="n") 


