# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
# Linear regression

# Water temperature = f(Air temperature
# Data from the Oir river, St Hilaire, France

# E. Rivot and Eric Parent
# June 2014
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------

library(stats4)
library(MASS)
rm(list = ls())



# -----------------------------------------------------------------------------
#                                  Load data
# -----------------------------------------------------------------------------
                                                                                
temp <- read.table(file = "T_Oir_StHilaire_87_02.txt", header = TRUE, sep = "")
temp <- temp[,c(5,6)]

head(temp)
dim(temp)
summary(temp)


# Indices to select data for the time interval 1st June - 30 Sept - 1987
summer1.b <- 151
summer1.e <- 273

# Indices to select data for the time interval 1st June - 30 Sept - 1987
summer2.b <- 1978
summer2.e <- 2099


# Data for fitting the model
data <-  temp[summer1.b:summer1.e,]

# Data for prediction (summer 2002)
data.pred <- temp[summer2.b:summer2.e,]


n <- dim(data)[1]



# -----------------------------------------------------------------------------
#                                 Plot the data 
# -----------------------------------------------------------------------------

# Time series - Summer 1987 only
# ------------------------------

windows()
par(mfrow = c(1,1))
plot(	1:dim(data)[1],data$Ta, col="red", lwd=1, type = "l", lty = 1, 
		ylab = "Temp (C)", xlab = "Day", main = "Oir River - June-Sept 1987"	)
points(1:dim(data)[1],data$Tw, col="blue", lwd=1, type = "l", lty = 1)
# abline(h=20, col="black", lwd=1, lty = 2)
# abline(v=c(31,61), col = "green", lty = 1, lwd = 2)
legend(	legend = c("TC Air","TC Water"),
 		col = c("red","blue"), 	
 		lty = c(2,1), lwd = c(2,2), x = "topright", cex = 1, bty ="n") 


# Pairwise correlation
# ------------------------------

windows()
plot(data$Ta, data$Tw,
	pch=21, cex=0.5, col = "red",  xlab = "Temp. air", ylab = "Temp. eau",
	main = "T air - T water")




# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
# 		            Linear Regression using lm()
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------

reg1 <- lm(formula  =  Tw  ~  Ta,   data = data)

summary(reg1)

# R
summary(reg1)$r.squared

# Confidence intervals
confint(reg1, level = 0.95)			#  level 95%
confint(reg1, level = 0.50)			#  level 50%

# covariance matrix
summary(reg1)$cov



# Plot the regression
# -----------------------------------------------------------------------------

windows()
par(mfrow = c(1,1))
plot(	data$Ta, data$Tw, pch = 21, col = "red", 
		xlab = "TC Air", ylab = "TC Water", main = "Oir River - June-Sept 1987",
		xlim = c(5,25), ylim = c(10,20))
abline(reg1, lty=1, lwd=2)
#  Print equation
text( x=7,y=18, pos=4, as.expression(substitute(y==a*x+b,
      list(a=round(coef(reg1)[2],2), b=round(coef(reg1)[1],2)))))
#  Print R
text( x=7, y=17, pos=4, as.expression( substitute( R==r,
      list(r=round(summary(reg1)$r.squared,2)) ) ) )

# Print confidence interval
pred.95 <- predict.lm(object = reg1, interval = "confidence", level = 0.95)
points(data$Ta, pred.95[,2], type = "l", col = "blue", lwd=2, lty=2)
points(data$Ta, pred.95[,3], type = "l", col = "blue", lwd=2, lty=2)


# Plot residuals and ACF
# --------------------------
windows() 
plot(reg1$res, type = "l", main = "Time series of residuals")
windows()
acf(reg1$res, main = "Autocorrelation")
acf(reg1$res)$acf[2]




# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#                        Predictions & Uncertainty
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
				         
time <- 1:dim(data.pred)[1]
time.inv <- dim(data.pred)[1]:1

level1=0.5
level2=0.95

windows()
plot(	time, data.pred$Tw, type = "n", col = "blue", ylim = c(10,22), lwd = 3, main = "Prediction TC Air - Summer 2002")

pred1 <- predict.lm(	object = reg1, newdata = data.pred, interval = "prediction", 	
				level = level1, type = "response")
pred2 <- predict.lm(	object = reg1, newdata = data.pred, interval = "prediction", 	
				level = level2, type = "response")

polygon(c(time,time.inv),c(pred2[,2],pred2[,3][dim(data.pred)[1]:1]),col="grey80",border=NA)
polygon(c(time,time.inv),c(pred1[,2],pred1[,3][dim(data.pred)[1]:1]),col="grey60",border=NA)
points(time, pred1[,1], type = "l", col = "black", lwd=2, lty=1)

points( time, data.pred$Tw, type = "l", col = "blue", lwd = 3)





# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#                          Likelihood profile
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------

#  Create a grid

range.a <- 0.20
nb.a <- 100
mod.a <- coef(reg1)[2]
min.a <- mod.a - range.a*abs(mod.a)
max.a <- mod.a + range.a*abs(mod.a)
a <- seq(from=min.a, to = max.a, length.out=nb.a)

range.b <- 0.20
nb.b <- 100
mod.b <- coef(reg1)[1]
min.b <- mod.b - range.b*abs(mod.b)
max.b <- mod.b + range.b*abs(mod.b)
b <- seq(from=min.b, to = max.b, length.out=nb.b)

dim.grid <- c(length(a),length(b), length(a)*length(b))
dim.grid
pas.a <- a[2]-a[1]
pas.b <- b[2]-b[1]


# Likelihood profile
# ----------------------------------

windows()
par(mfrow = c(1,2))

reg <- lm(formula  =  Tw  ~  Ta,   data = data)
var <- var(resid(reg))
n <- length(resid(reg))

loglik <- matrix(nrow = length(a), ncol = length(b))

for (i in 1:length(a)) { 
for (j in 1:length(b)) {
loglik[i,j] <-  - 0.5*n*log(var*2*pi) - sum( 0.5*(1/var)*(data$Tw - a[i]*data$Ta - b[j])^2)
}}
lik <- exp(loglik + abs(max(loglik)))

persp(	x = a, y = b, lik, 
		xlim = range(a), ylim = range(b), xlab = "alpha", ylab = "beta", zlab = "", main = "Likelihood (scaled)",
		ticktype = "detailed", nticks = 5, theta = 0, col = "red")

contour(	x = a, y = b, lik , nlevels = 5, col = "blue", lwd = 2, 
		xlab = "alpha", ylab = "beta", main = "Likelihood contours") 







